# Copyright (c) 2007-2012 Christoph Haas <email@christoph-haas.de>
# See the file LICENSE for copying permission.

"""Enhances the paginate.Page class to work with SQLAlchemy objects"""

import paginate


class SqlalchemyOrmWrapper(object):
    """Wrapper class to access elements of an SQLAlchemy ORM query result."""

    def __init__(self, obj):
        self.obj = obj

    def __getitem__(self, range):
        if not isinstance(range, slice):
            raise Exception("__getitem__ without slicing not supported")
        return self.obj[range]

    def __len__(self):
        return self.obj.count()


class SqlalchemyOrmPage(paginate.Page):
    """A pagination page that deals with SQLAlchemy ORM objects.
    
    See the documentation on paginate.Page for general information on how to work
    with instances of this class."""

    # This class just subclasses paginate.Page which contains all the functionality.
    # It just instantiates the class with a "wrapper_class" argument telling it how the
    # collection can be accessed.
    def __init__(self, *args, **kwargs):
        super(SqlalchemyOrmPage, self).__init__(
            *args, wrapper_class=SqlalchemyOrmWrapper, **kwargs)


def sql_wrapper_factory(db_session=None):
    class SqlalchemySelectWrapper(object):
        """Wrapper class to access elements of an SQLAlchemy SELECT query."""

        def __init__(self, obj):
            self.obj = obj
            self.db_session = db_session

        def __getitem__(self, range):
            if not isinstance(range, slice):
                raise Exception("__getitem__ without slicing not supported")
            # value for offset
            offset_v = range.start
            limit = range.stop - range.start
            select = self.obj.limit(limit).offset(offset_v)
            return self.db_session.execute(select).fetchall()

        def __len__(self):
            return self.db_session.execute(self.obj.alias().count()).scalar()

    return SqlalchemySelectWrapper


class SqlalchemySelectPage(paginate.Page):
    """A pagination page that deals with SQLAlchemy Select objects.
    
    See the documentation on paginate.Page for general information on how to work
    with instances of this class."""

    # This class just subclasses paginate.Page which contains all the functionality.
    # It just instantiates the class with a "wrapper_class" argument telling it how the
    # collection can be accessed.
    def __init__(self, db_session, *args, **kwargs):
        """sqlalchemy_connection: SQLAlchemy connection object"""
        wrapper = sql_wrapper_factory(db_session)
        super(SqlalchemySelectPage, self).__init__(
            *args, wrapper_class=wrapper, **kwargs)



from setuptools import setup, find_packages

import sys, os

setup(
    name='paginate_sqlalchemy',
    version='0.3.1',
    description="Extension to paginate.Page that supports SQLAlchemy queries",
    long_description="""
        This module helps divide up large result sets into pages or chunks.
        The user gets displayed one page at a time and can navigate to other pages.
        It is especially useful when developing web interfaces and showing the
        users only a selection of information at a time.

        This module uses and extends the functionality of the paginate module to
        support SQLAlchemy queries.
        """,

    # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ],
    keywords='pagination paginate sqlalchemy',
    author='Christoph Haas',
    author_email='email@christoph-haas.de',
    maintainer='Luke Crooks',
    maintainer_email='luke@pumalo.org',
    install_requires=[
        "sqlalchemy>=0.8.3",
        "paginate>=0.4"
        ],
    url='https://github.com/Pylons/paginate_sqlalchemy',
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    entry_points=""" """,
)
